let portFromCS;
function connected(p) {
  portFromCS = p;
}
browser.runtime.onConnect.addListener(connected);


let messageTypeKotlinCall = "kotlin_call";

let kotlinCallCtr = 0;
let kotlinCalls = {};


function kotlinCall(funcName, params) {
  return new Promise((resolve, reject) => {
    kotlinCallCtr++;
    portFromCS.postMessage({
      type: messageTypeKotlinCall,
      funcName: funcName,
      params: params,
      callId: kotlinCallCtr,
    });

    kotlinCalls[kotlinCallCtr] = {
      resolve: resolve,
      reject: reject,
    };
  });
}



browser.runtime.onMessage.addListener((message) => {
  nativeLog(`Background Received: ${ message}, calls: ${kotlinCalls}`);
  switch (message.type) {
    case messageTypeKotlinCall:
      try {
        if (message.result) {
          let result = message.result;
          kotlinCalls[message.callId].resolve(result);
        } else {
          let error = message.error;
          kotlinCalls[message.callId].reject(error);
        }
        delete kotlinCalls[message.callId];
      } catch (e) {
        nativeLog("error: " + e.toString());
      }
      break;
    default:
      nativeLog("Background: unknown message type: " + message.type);
      break;
  }
});


function nativeLog(s) {
  try{
    portFromCS.postMessage(s);
  }catch(e){
    console.log(s);
  }
}



async function redirect(requestDetails) {
  nativeLog("redirect: " + requestDetails.url);
  if(!portFromCS){
    nativeLog("portFromCS not connected");
    return;
  }
  if(await shouldBlockRequest(requestDetails.url)){
    nativeLog("Blocking: " + requestDetails.url);
    return {
      redirectUrl: "k",
    }
  }

  return {
    redirectUrl: requestDetails.url,
  };
}



try {
  browser.webRequest.onBeforeRequest.addListener(
    redirect,
    {
      urls: ["<all_urls>"],
    },
    ["blocking"]
  );
} catch (e) {
  nativeLog("onBeforeRequest: error: " + browser);
  nativeLog("onBeforeRequest: error: " + e);
}

async function shouldBlockRequest(url){
  return kotlinCall("shouldBlockRequest", [url]);
}