// Based on: https://github.com/erkserkserks/h264ify/blob/master/src/inject/inject.js
(function () {
  const block60fps = true;

  // return a custom MIME type checker that can defer to the original function
  function makeModifiedTypeChecker(origChecker) {
    // Check if a video type is allowed
    return function (type) {
      if (type === undefined) return "";
      const disallowed_types = ["webm", "vp8", "vp9", "av01"];
      // If video type is in disallowed_types, say we don't support them
      for (let i = 0; i < disallowed_types.length; i++) {
        if (type.indexOf(disallowed_types[i]) !== -1) return "";
      }

      if (block60fps) {
        const match = /framerate=(\d+)/.exec(type);
        if (match && match[1] > 30) return "";
      }

      // Otherwise, ask the browser
      return origChecker(type);
    };
  }

  function overrideSupportedCodecs() {
    // Override video element canPlayType() function
    const videoElem = document.createElement("video");
    const origCanPlayType = videoElem.canPlayType.bind(videoElem);
    videoElem.__proto__.canPlayType = makeModifiedTypeChecker(origCanPlayType);

    // Override media source extension isTypeSupported() function
    const mse = window.MediaSource;
    // Check for MSE support before use
    if (mse === undefined) return;
    const origIsTypeSupported = mse.isTypeSupported.bind(mse);
    mse.isTypeSupported = makeModifiedTypeChecker(origIsTypeSupported);
  }

  overrideSupportedCodecs();
})();
